#ifndef _SERVER_H
#define _SERVER_H

#include <functional>
#include <netinet/in.h>
#include <unistd.h>
#include <vector>

#define INPUT_BUFFER_SIZE 1000

class Server {
public:
  typedef std::function<void(uint16_t, char *)> receiveCallbackType;
  typedef std::function<void(uint16_t)> newConnectionCallbackType;
  typedef std::function<void(uint16_t)> disconnectCallbackType;

public:
  Server();
  Server(const int &);
  Server(const Server &) {}
  virtual ~Server();

  void init();
  void run();
  void stop();

  // callback setters
  void onConnect(newConnectionCallbackType cb);
  void onInput(receiveCallbackType cb);
  void onDisconnect(disconnectCallbackType cb);

  uint16_t sendMessage(int source_fd, const char *messageBuffer);
  uint16_t sendMessage(int source_fd, char *messageBuffer);

private:
  // file descriptor sets
  fd_set masterfds;
  fd_set tmpfds;

  // track the maximum fd value
  uint16_t maxfd;

  int mastersocket_fd;
  int tempsocket_fd;

  struct sockaddr_storage client_addr;
  struct sockaddr_in server_addr;

  char input_buffer[INPUT_BUFFER_SIZE];

  char remote_ip[INET6_ADDRSTRLEN];

  newConnectionCallbackType newConnectionCallback;
  receiveCallbackType receiveCallback;
  disconnectCallbackType disconnectCallback;

  // function prototypes
  void setup(int port);
  void initializeSocket();
  void bindSocket();
  void startListen();
  void handleNewConnection();
  void recvInputFromExisting(int fd);
};
#endif // !_SERVER_H
